@@ -1,7 +1,7 @@
 # ============================================================
 #
 # Copyright (C) 2010 by Johannes Wienke <jwienke at techfak dot uni-bielefeld dot de>
-# Copyright (C) 2011, 2012, 2013 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
+# Copyright (C) 2011, 2012, 2013, 2014 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
 #
 # This file may be licensed under the terms of the
 # GNU Lesser General Public License Version 3 (the ``LGPL''),
@@ -35,7 +35,6 @@ L{createListener}, L{createServer} and L{createRemoteServer}.
 @author: jmoringe
 """
 
-import abc
 import uuid
 import copy
 import logging
@@ -152,6 +151,8 @@ class ParticipantConfig (object):
      - Employed transport mechanisms
        - Their configurations (e.g. port numbers)
        - Associated converters
+     - Whether introspection should be enabled for the participant
+       (enabled by default)
 
     @author: jmoringe
     """
@@ -169,7 +170,7 @@ class ParticipantConfig (object):
         """
         def __init__(self, name, options={}, converters=None):
             self.__name = name
-            self.__enabled = options.get('enabled', '0') in ('1', 'true', 'yes')
+            self.__enabled = options.get('enabled', '0') in ('1', 'true', 'yes') # TODO ugly
 
             # Extract freestyle options for the transport.
             self.__options = dict([ (key, value) for (key, value) in options.items()
@@ -214,14 +215,25 @@ class ParticipantConfig (object):
 
         options = property(getOptions)
 
+        def __deepcopy__(self, memo):
+            result                  = copy.copy(self)
+            result.__converters     = copy.deepcopy(self.__converters, memo)
+            result.__converterRules = copy.deepcopy(self.__converterRules, memo)
+            result.__options        = copy.deepcopy(self.__options, memo)
+            return result
+
         def __str__(self):
-            return ('ParticipantConfig.Transport[%s, enabled = %s,  converters = %s, converterRules = %s, options = %s]'
+            return ('ParticipantConfig.Transport[%s, enabled = %s, converters = %s, converterRules = %s, options = %s]'
                     % (self.__name, self.__enabled, self.__converters, self.__converterRules, self.__options))
 
         def __repr__(self):
             return str(self)
 
-    def __init__(self, transports=None, options=None, qos=None):
+    def __init__(self,
+                 transports    = None,
+                 options       = None,
+                 qos           = None,
+                 introspection = False):
         if transports is None:
             self.__transports = {}
         else:
@@ -237,6 +249,8 @@ class ParticipantConfig (object):
         else:
             self.__qos = qos
 
+        self.__introspection = introspection
+
     def getTransports(self, includeDisabled=False):
         return [ t for t in self.__transports.values()
                  if includeDisabled or t.isEnabled() ]
@@ -249,8 +263,28 @@ class ParticipantConfig (object):
     def getQualityOfServiceSpec(self):
         return self.__qos
 
+    def setQualityOfServiceSpec(self, newValue):
+        self.__qos = newValue
+
+    qualityOfServiceSpec = property(getQualityOfServiceSpec,
+                                    setQualityOfServiceSpec)
+
+    def getIntrospection(self):
+        return self.__introspection
+
+    def setIntrospection(self, newValue):
+        self.__introspection = newValue
+
+    introspection = property(getIntrospection, setIntrospection)
+
+    def __deepcopy__(self, memo):
+        result              = copy.copy(self)
+        result.__transports = copy.deepcopy(self.__transports, memo)
+        result.__options    = copy.deepcopy(self.__options, memo)
+        return result
+
     def __str__(self):
-        return 'ParticipantConfig[%s %s]' % (self.__transports.values(), self.__options)
+        return 'ParticipantConfig[%s, options = %s, qos = %s, introspection = %s]' % (self.__transports.values(), self.__options, self.__qos, self.__introspection)
 
     def __repr__(self):
         return str(self)
@@ -272,6 +306,11 @@ class ParticipantConfig (object):
             transportOptions = dict(sectionOptions('transport.%s' % transport))
             if transportOptions:
                 result.__transports[transport] = clazz.Transport(transport, transportOptions)
+
+        # Introspection options
+        introspectionOptions = dict(sectionOptions('introspection'))
+        result.__introspection = introspectionOptions.get('enabled', '1') in [ '1', 'true', 'yes' ] # TODO ugly
+
         return result
 
     @classmethod
@@ -371,7 +410,10 @@ class ParticipantConfig (object):
 
         import util
 
-        defaults = {"transport.socket.enabled": "1"}
+        defaults = {
+            'transport.socket.enabled': '1',
+            'introspection.enabled':    '1'
+        }
         if platform.system() == 'Windows':
             partial = clazz.__fromFile("c:\\rsb.conf", defaults)
         else:
@@ -440,7 +482,7 @@ class Scope(object):
     """
 
     __COMPONENT_SEPARATOR = "/"
-    __COMPONENT_REGEX = re.compile("^[a-zA-Z0-9]+$")
+    __COMPONENT_REGEX = re.compile("^[-_a-zA-Z0-9]+$")
 
     @classmethod
     def ensureScope(cls, thing):
@@ -1110,8 +1152,8 @@ class Event(object):
         if len(printData) > 100:
             printData = printData[:100] + '...'
         printData = ''.join(['\\x%x' % ord(c) if ord(c) < 32 else c for c in printData])
-        return "%s[id = %s, scope = '%s', data = '%s', type = '%s', metaData = %s, causes = %s]" \
-            % ("Event", self.__id, self.__scope, printData, self.__type, self.__metaData, self.__causes)
+        return "%s[id = %s, scope = '%s', data = '%s', type = '%s', method = '%s', metaData = %s, causes = %s]" \
+            % ("Event", self.__id, self.__scope, printData, self.__type, self.__method, self.__metaData, self.__causes)
 
     def __repr__(self):
         return self.__str__()
@@ -1125,6 +1167,35 @@ class Event(object):
     def __neq__(self, other):
         return not self.__eq__(other)
 
+class Hook (object):
+    """
+    A mutable collection of callback functions that can be called
+    together.
+
+    @author: jmoringe
+    """
+
+    def __init__(self):
+        self.__lock     = threading.RLock()
+        self.__handlers = []
+
+    def run(self, *args, **kwargs):
+        with self.__lock:
+            for handler in self.__handlers:
+                handler(*args, **kwargs)
+
+    def addHandler(self, handler):
+        with self.__lock:
+            self.__handlers.append(handler)
+
+    def removeHandler(self, handler):
+        with self.__lock:
+            self.__handlers.remove(handler)
+
+participantCreationHook = Hook()
+
+participantDestructionHook = Hook()
+
 class Participant(object):
     """
     Base class for specialized bus participant classes. Has a unique
@@ -1132,7 +1203,7 @@ class Participant(object):
 
     @author: jmoringe
     """
-    def __init__(self, scope):
+    def __init__(self, scope, config):
         """
         Constructs a new Participant. This should not be done by
         clients.
@@ -1140,11 +1211,15 @@ class Participant(object):
         @param scope: scope of the bus channel.
         @type scope: Scope or accepted by Scope constructor
 
+        @param config: Configuration that the participant should use
+        @type config: ParticipantConfig
+
         See L{createListener}, L{createInformer}, L{createServer},
-        L{createRemoteServer}, L{createService}
+        L{createRemoteServer}
         """
         self.__id = uuid.uuid4()
         self.__scope = Scope.ensureScope(scope)
+        self.__config = config
 
     def getId(self):
         return self.__id
@@ -1162,13 +1237,21 @@ class Participant(object):
 
     scope = property(getScope, setScope)
 
-    @abc.abstractmethod
+    def getConfig(self):
+        return self.__config
+
+    config = property(getConfig)
+
+    def activate(self):
+        pass
+
     def deactivate(self):
         """
-        Deactivates a participant by setting tearing down all connection logic.
-        This needs to be called in case you want to ensure that programs can
-        terminate correctly.
+        Deactivates a participant by tearing down all connection
+        logic. This needs to be called in case you want to ensure
+        that programs can terminate correctly.
         """
+        participantDestructionHook.run(self)
 
     def __enter__(self):
         return self
@@ -1223,10 +1306,10 @@ class Informer(Participant):
     Event-sending part of the communication pattern.
 
     @author: jwienke
+    @author: jmoringe
     """
 
-    def __init__(self, scope, theType,
-                 config       = None,
+    def __init__(self, scope, config, dataType,
                  configurator = None):
         """
         Constructs a new L{Informer} that publishes L{Event}s carrying
@@ -1234,30 +1317,32 @@ class Informer(Participant):
 
         @param scope: scope of the informer
         @type scope: Scope or accepted by Scope constructor
-        @param theType: A Python object designating the type of objects
-                        that will be sent via the new informer. Instances
-                        of subtypes are permitted as well.
-        @type theType: type
+        @param config: The configuration that should be used by this
+                       L{Informer}.
+        @type config: ParticipantConfig
+        @param dataType: A Python object designating the type of
+                         objects that will be sent via the new
+                         L{Informer}. Instances of subtypes are
+                         permitted as well.
+        @type dataType: type
         @param configurator: Out route configurator to manage sending
                              of events through out connectors.
         @todo: maybe provide an automatic type identifier deduction for default
                types?
+        @see: L{createInformer}
         """
-        super(Informer, self).__init__(scope)
+        super(Informer, self).__init__(scope, config)
 
         self.__logger = getLoggerByClass(self.__class__)
 
         # TODO check that type can be converted
-        self.__type           = theType
+        self.__type           = dataType
         self.__sequenceNumber = 0
         self.__configurator   = None
 
         self.__active         = False
         self.__mutex          = threading.Lock()
 
-        if config is None:
-            config = getDefaultParticipantConfig()
-
         if configurator:
             self.__configurator = configurator
         else:
@@ -1331,6 +1416,8 @@ class Informer(Participant):
 
             self.__active = True
 
+        self.activate()
+
     def deactivate(self):
         with self.__mutex:
             if not self.__active:
@@ -1342,15 +1429,17 @@ class Informer(Participant):
 
             self.__configurator.deactivate()
 
+        super(Informer, self).deactivate()
+
 class Listener(Participant):
     """
     Event-receiving part of the communication pattern
 
     @author: jwienke
+    @author: jmoringe
     """
 
-    def __init__(self, scope,
-                 config            = None,
+    def __init__(self, scope, config,
                  configurator      = None,
                  receivingStrategy = None):
         """
@@ -1359,11 +1448,15 @@ class Listener(Participant):
         @param scope: The scope of the channel in which the new
                       listener should participate.
         @type scope: Scopeor or accepted by Scope constructor
+        @param config: The configuration that should be used by this
+                        L{Listener}.
+        @type config: ParticipantConfig
         @param configurator: An in route configurator to manage the
                              receiving of events from in connectors
                              and their filtering and dispatching.
+        @see: L{createListener}
         """
-        super(Listener, self).__init__(scope)
+        super(Listener, self).__init__(scope, config)
 
         self.__logger = getLoggerByClass(self.__class__)
 
@@ -1373,9 +1466,6 @@ class Listener(Participant):
         self.__active       = False
         self.__mutex        = threading.Lock()
 
-        if config is None:
-            config = getDefaultParticipantConfig()
-
         if configurator:
             self.__configurator = configurator
         else:
@@ -1404,6 +1494,7 @@ class Listener(Participant):
 
             self.__active = True
 
+        self.activate()
 
     def deactivate(self):
         with self.__mutex:
@@ -1416,6 +1507,8 @@ class Listener(Participant):
 
             self.__active = False
 
+        super(Listener, self).deactivate()
+
     def addFilter(self, theFilter):
         """
         Appends a filter to restrict the events received by this listener.
@@ -1499,41 +1592,62 @@ def setDefaultParticipantConfig(config):
     global __defaultParticipantConfig
     __defaultParticipantConfig = config
 
-def createListener(scope, config = None):
-    """
-    Creates a new Listener for the specified scope.
+def createParticipant(clazz, scope, config, parent = None, **kwargs):
+    if config is None:
+        config = getDefaultParticipantConfig()
 
-    @param scope: the scope of the new Listener. Can be a Scope object or a string.
-    @type scope: Scope or accepted by Scope constructor
-    @return: a new Listener object.
-    """
-    return Listener(scope, config)
+    if config.introspection:
+        import rsb.introspection
 
-def createInformer(scope, config=None, dataType=object):
-    """
-    Creates a new Informer in the specified scope.
+    participant = clazz(scope, config = config, **kwargs)
+    participantCreationHook.run(participant, parent = parent)
+    return participant
 
-    @param scope: The scope of the new Informer. Can be a Scope object
-                  or a string.
-    @type scope: Scope or accepted by Scope constructor
-    @param dataType: the string representation of the data type used
-                     to select converters
-    @return: a new Informer object.
+def createListener(scope, config = None, parent = None, **kwargs):
     """
-    return Informer(scope, dataType, config)
+    Creates and returns a new L{Listener} for B{scope}.
 
-def createService(scope):
+    @param scope: the scope of the new L{Listener}. Can be a L{Scope} object or a string.
+    @type scope: Scope or accepted by L{Scope} constructor
+    @param config: The configuration that should be used by this
+                   L{Listener}.
+    @type config: ParticipantConfig
+    @param parent: C{None} or the L{Participant} which should be
+                   considered the parent of the new L{Listener}.
+    @type  parent: Participant or NoneType
+    @return: a new L{Listener} object.
     """
-    Creates a Service object operating on the given scope.
-    @param scope: parent-scope of the new service. Can be a Scope
+    return createParticipant(Listener, scope, config, parent,
+                             **kwargs)
+
+def createInformer(scope, config = None, parent = None, dataType = object,
+                   **kwargs):
+    """
+    Creates and returns a new L{Informer} for B{scope}.
+
+    @param scope: The scope of the new L{Informer}. Can be a L{Scope}
                   object or a string.
-    @type scope: Scope or accepted by Scope constructor
-    @return: new Service object
+    @type scope: Scope or accepted by L{Scope} constructor
+    @param config: The configuration that should be used by this
+                   L{Informer}.
+    @type config: ParticipantConfig
+    @param parent: C{None} or the L{Participant} which should be
+                   considered the parent of the new L{Informer}.
+    @type  parent: Participant or NoneType
+    @param dataType: A Python object designating the type of
+                     objects that will be sent via the new
+                     L{Informer}. Instances of subtypes are
+                     permitted as well.
+    @type dataType: type
+    @return: a new L{Informer} object.
     """
-    raise RuntimeError, "not implemented"
+    return createParticipant(Informer, scope, config, parent,
+                             dataType = dataType,
+                             **kwargs)
 
-def createServer(scope, object = None, expose = None, methods = None,
-                 config = None):
+def createLocalServer(scope, config = None, parent = None,
+                      object = None, expose = None, methods = None,
+                      **kwargs):
     """
     Create and return a new L{LocalServer} object that exposes its
     methods under B{scope}.
@@ -1544,10 +1658,13 @@ def createServer(scope, object = None, expose = None, methods = None,
 
     @param scope: The scope under which the newly created server
                   should expose its methods.
-    @type scope: Scope or accepted by Scope constructor
-    @param config: The transport configuration that should be used
-                   for communication performed by this server.
+    @type scope: Scope or accepted by L{Scope} constructor
+    @param config: The configuration that should be used by this
+                   server.
     @type config: ParticipantConfig
+    @param parent: C{None} or the L{Participant} which should be
+                   considered the parent of the new server.
+    @type  parent: Participant or NoneType
     @param object: An object the methods of which should be exposed
                    via the newly created server. Has to be supplied in
                    combination with the expose keyword parameter.
@@ -1572,7 +1689,9 @@ def createServer(scope, object = None, expose = None, methods = None,
 
     # Create the server object and potentially add methods.
     import rsb.patterns
-    server = rsb.patterns.LocalServer(scope, config)
+    server = createParticipant(rsb.patterns.LocalServer,
+                               scope, config, parent,
+                               **kwargs)
     if object and expose:
         methods = [ (name, getattr(object, name), requestType, replyType)
                     for (name, requestType, replyType) in expose ]
@@ -1581,7 +1700,7 @@ def createServer(scope, object = None, expose = None, methods = None,
             server.addMethod(name, func, requestType, replyType)
     return server
 
-def createRemoteServer(scope, config = None):
+def createRemoteServer(scope, config = None, parent = None, **kwargs):
     """
     Create a new L{RemoteServer} object for a remote server that
     provides its methods under B{scope}.
@@ -1593,7 +1712,24 @@ def createRemoteServer(scope, config = None):
     @param config: The transport configuration that should be used
                    for communication performed by this server.
     @type config: ParticipantConfig
+    @param parent: C{None} or the L{Participant} which should be
+                   considered the parent of the new server.
+    @type  parent: Participant or NoneType
     @rtype: rsb.patterns.RemoteServer
     """
     import rsb.patterns
-    return rsb.patterns.RemoteServer(scope, config)
+    return createParticipant(rsb.patterns.RemoteServer, scope, config,
+                             **kwargs)
+
+def createServer(scope, config = None, parent = None,
+                 object = None, expose = None, methods = None,
+                 **kwargs):
+
+    """
+    Like L{createLocalServer}.
+
+    @deprecated: Use L{createLocalServer} instead.
+    """
+    return createLocalServer(scope, config, parent,
+                             object = object, expose = expose, methods = methods,
+                             **kwargs)
